@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Part of the scorify package
-# Copyright 2014 Board of Regents of the University of Wisconsin System
+# Copyright (c) 2020 Board of Regents of the University of Wisconsin System
 
 """
 Reads data files (or CSV objects) into datafile objects.
@@ -9,41 +9,61 @@ Datafiles are iterable and indexable by column name. When reading, you pass
 in a scoresheet.LayoutSection, which tells you where data and header sections
 are.
 """
+from __future__ import absolute_import
+import warnings
 
 from scorify.errors import HaystackError
 
 
 class Datafile(object):
-    def __init__(self, lines, layout_section):
+    def __init__(self, lines, layout_section, rename_section):
         self.lines = lines
         self.layout_section = layout_section
+        self.rename_section = rename_section
         self.header = []
+        self.keep = []
         self.data = []
         super(Datafile, self).__init__()
 
     def read(self):
         self.header = []
         self.data = []
+        self.keep = []
         for line_num, line in enumerate(self.lines):
             # Since we assume layout_section is valid, we only care about
-            # header and skip lines -- everything else must be data.
+            # header and skip and keep lines -- everything else must be data.
             line_type = ''
             if line_num < len(self.layout_section.directives):
                 line_type = self.layout_section.directives[line_num].info
             if line_type == 'skip':
                 continue
             if line_type == 'header':
-                self.header = [h.strip() for h in line]
+                self.header = [
+                    self.rename_section.map_name(h.strip()) for h in line]
+                # Warn if header contains duplicates
+                seen_set = set()
+                duplicates = set(x for x in self.header if x in seen_set or seen_set.add(x))
+                if len(duplicates) > 0:
+                    warnings.warn("Duplicates in header: " + str(duplicates), UserWarning)
+            elif line_type == 'keep':
+                self.append_keep(line)
             else:
                 self.append_data(line)
 
-    def append_data(self, data):
+    def pad_data(self, data):
         # Force lines of funny length to be the header's length
         len_diff = len(self.header) - len(data)
         padding = [''] * len_diff
-        full_line = data + padding
+        return data + padding
+
+    def append_data(self, data):
+        full_line = self.pad_data(data)
         self.data.append(dict(zip(self.header, full_line)))
 
+    def append_keep(self, data):
+        full_line = self.pad_data(data)
+        self.keep.append(dict(zip(self.header, full_line)))
+
     def apply_exclusions(self, exclusion_section):
         new_data = []
         for row in self.data:
@@ -51,7 +71,7 @@ class Datafile(object):
                 exclude = any([e.excludes(row) for e in exclusion_section])
             except KeyError as exc:
                 raise ExclusionError(
-                    "data columns", exc.message, self.data.header)
+                    "data columns", str(exc), self.data.header)
             if not exclude:
                 new_data.append(row)
         self.data = new_data
