@@ -1,15 +1,17 @@
 # -*- coding: utf-8 -*-
 # Part of the scorify package
-# Copyright 2014 Board of Regents of the University of Wisconsin System
+# Copyright (c) 2020 Board of Regents of the University of Wisconsin System
+from __future__ import absolute_import
 
-import directives
-import mappings
+from scorify import directives
+from scorify import mappings
 
 
 class Scoresheet(object):
     def __init__(self):
         self.errors = []
         self.layout_section = LayoutSection()
+        self.rename_section = RenameSection()
         self.exclude_section = ExcludeSection()
         self.transform_section = TransformSection()
         self.score_section = ScoreSection()
@@ -26,7 +28,7 @@ class Reader(object):
     def __init__(self, data=None):
         """
         data should be a csv.Reader-like object. Notably, it should be
-        iterable, return a list per iteration, and support line_number.
+        iterable, return a list per iteration, and support line_num.
         """
         self.data = data
 
@@ -44,6 +46,7 @@ class Reader(object):
             sheet = Scoresheet()
         section_map = {
             'layout': sheet.layout_section,
+            'rename': sheet.rename_section,
             'exclude': sheet.exclude_section,
             'transform': sheet.transform_section,
             'score': sheet.score_section,
@@ -69,7 +72,7 @@ class Reader(object):
                 mappings.MappingError
             ) as exc:
                 sheet.add_error("Line {0}: {1}".format(
-                    self.data.line_num, exc.message))
+                    self.data.line_num, str(exc)))
         if not sheet.layout_section.is_valid():
             for err in sheet.layout_section.errors:
                 sheet.add_error(err)
@@ -140,6 +143,36 @@ class LayoutSection(Section):
         self.append_directive(directives.Layout(string_list[0]))
 
 
+class RenameSection(Section):
+    def __init__(self, directives=None):
+        self.mapper = {}
+        super(RenameSection, self).__init__(directives)
+
+    def append_from_strings(self, string_list):
+        if len(string_list) < 2:
+            raise directives.DirectiveError(
+                "rename needs an original and new column name")
+        self.append_directive(
+            directives.Rename(string_list[0], string_list[1]))
+
+    def append_directive(self, directive):
+        self.__raise_on_conflict(directive)
+        self.mapper[directive.original_name] = directive.new_name
+        super(RenameSection, self).append_directive(directive)
+
+    def map_name(self, name):
+        # Default to the original.
+        return self.mapper.get(name, name)
+
+    def __raise_on_conflict(self, directive):
+        conflict = next(
+            (d for d in self.directives if directive.conflicts_with(d)),
+            None)
+        if conflict:
+            raise SectionError(
+                "%s conflicts with %s" % (directive, conflict))
+
+
 class ExcludeSection(Section):
     def __init__(self, directives=None):
         super(ExcludeSection, self).__init__(directives)
@@ -197,13 +230,16 @@ class ScoreSection(Section):
         col_name = string_list[0]
         measure_name = ''
         transform = ''
+        output_name = None
         if len(string_list) > 1:
             measure_name = string_list[1]
         if len(string_list) > 2:
             transform = string_list[2]
+        if len(string_list) > 3:
+            output_name = string_list[3]
 
         self.append_directive(
-            directives.Score(col_name, measure_name, transform))
+            directives.Score(col_name, measure_name, transform, output_name))
 
     def append_directive(self, directive):
         column = directive.column
