@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
 # Part of the scorify package
-# Copyright 2014 Board of Regents of the University of Wisconsin System
+# Copyright (c) 2020 Board of Regents of the University of Wisconsin System
 
 from __future__ import with_statement
 
 import pytest
 
 from scorify.mappings import (
-    Mapping, Identity, LinearMapping, DiscreteMapping, MappingError)
+    Mapping, Identity, LinearMapping, DiscreteMapping, MappingError,
+    PassthroughMapping)
 
 
 def test_identity():
@@ -59,11 +60,11 @@ def test_linear_mapping_from_string():
 
 
 def test_discrete_mapping_from_string():
-    m = DiscreteMapping.from_string('string_map("1":"f","2":"m")')
+    m = DiscreteMapping.from_string('discrete_map("1":"f","2":"m")')
     assert m.map_dict == {"1": "f", "2": "m"}
     # And let's get trickier
     m = DiscreteMapping.from_string(
-        r'string_map("1\"": "f" , "2" :"m")')
+        r'discrete_map("1\"": "f" , "2" :"m")')
     assert m.map_dict == {r'1"': "f", "2": "m"}
 
 
@@ -74,9 +75,23 @@ def test_discrete_mapping_transform():
     assert m.transform("3") == ""
 
 
+def test_passthrough_mapping_from_string():
+    m = PassthroughMapping.from_string('passthrough_map("1":"f","2":"m")')
+    assert m.map_dict == {"1": "f", "2": "m"}
+
+
+def test_passthrough_mapping_transform():
+    m = PassthroughMapping({"1": "f", "2": "m"})
+    assert m.transform("1") == "f"
+    assert m.transform("3") == "3"
+
+
 def test_mapping_types():
     assert type(Mapping.from_string('')) == Identity
     assert type(Mapping.from_string('i')) == Identity
     assert type(Mapping.from_string('map(1:3,2:4)')) == LinearMapping
     assert type(
         Mapping.from_string('discrete_map("a":"b")')) == DiscreteMapping
+
+    assert type(
+        Mapping.from_string('passthrough_map("a":"b")')) == PassthroughMapping
