@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Part of the scorify package
-# Copyright 2014 Board of Regents of the University of Wisconsin System
+# Copyright (c) 2020 Board of Regents of the University of Wisconsin System
 
 import pytest
 
@@ -9,13 +9,15 @@ from scorify import scoresheet, directives
 
 @pytest.fixture
 def good_sample_csv():
-    import StringIO
+    import io
     import csv
-    return csv.reader(StringIO.StringIO("""
+    return csv.reader(io.StringIO("""
 layout,header
 layout,skip
  layout,data
 
+rename,happy_1,happy
+
 # This is a comment
 exclude,ppt,0000
 exclude,ppt,9999
@@ -33,6 +35,9 @@ score,gender,gender,gender
 measure,mean_happy,mean(happy)
 measure,mean_sad,mean(sad)
 measure,emo_vals,"join(happy, sad)"
+measure,happy_ratio,"ratio(mean_happy,mean_sad)"
+measure,min_emo,"min(happy, sad)"
+measure,max_happy,max(happy)
 """))
 
 
@@ -42,9 +47,10 @@ def test_successful_read(good_sample_csv):
     assert type(ss) == scoresheet.Scoresheet
     assert ss.errors == []
     assert len(ss.layout_section) == 3
+    assert len(ss.rename_section) == 1
     assert len(ss.transform_section) == 3
     assert len(ss.score_section) == 5
-    assert len(ss.measure_section) == 3
+    assert len(ss.measure_section) == 6
 
 
 def test_section_iterates():
@@ -97,6 +103,28 @@ def test_layout_section_with_string_ary():
         ls.append_from_strings(['foo'])
 
 
+def test_rename_section_with_string_list():
+    s = scoresheet.RenameSection()
+    s.append_from_strings(['foo', 'bar'])
+    assert len(s) == 1
+    assert s.map_name('foo') == 'bar'
+    assert s.map_name('baz') == 'baz'
+
+
+def test_rename_section_dupes():
+    s = scoresheet.RenameSection()
+    s.append_from_strings(["foo", "bar"])
+    # "foo" and "bar" are off-limits for any other renames
+    with pytest.raises(scoresheet.SectionError):
+        s.append_from_strings(["foo", "x"])
+    with pytest.raises(scoresheet.SectionError):
+        s.append_from_strings(["x", "foo"])
+    with pytest.raises(scoresheet.SectionError):
+        s.append_from_strings(["bar", "x"])
+    with pytest.raises(scoresheet.SectionError):
+        s.append_from_strings(["x", "bar"])
+
+
 def test_transform_section_dupes():
     s = scoresheet.TransformSection()
     d = directives.Transform("foo", "map(1:5, 1:5)")
